if (jira === undefined)
{
    var jira = {}
}
;
if (jira.widget === undefined)
{
    jira.widget = {}
}
;

// This will be defined by FireBug and not by anythign else.  Please dont leave the console.log messages in the code however
if (console === undefined)
{
    var console = {}
    console.log = function()
    {
    }
}

/**
 * A drop down menu button widget.
 *
 * @param linkedId - {String | HtmlElement} the linked element, clicking on which will drop down this menu
 * @param url {String} the url that provides JSON data of menu items
 * @param jiraContextPath {String} the web context path of jira (for imaegs and the like)
 * @param contextAlignment {Array}  the alignment of the menu relative to the linkedId.  Default is ["tr","br"]
 */
jira.widget.MenuButton = function(linkedId, url, jiraContextPath, contextAlignment)
{
    contextAlignment = contextAlignment || ["tr", "br"];

    //
    // this is the "throbber" that is shown during load up.  if things are fast, it is shown for a
    // very short time.  If things are slow it will be shown for longer.
    //
    var sBody = '<img src="' + jiraContextPath + '/images/throbber/wait.gif"/>';
    var that = this;
    var waitingOverlay = new YAHOO.widget.Overlay(linkedId + "_wait", {
        constraintoviewport : true,
        visible : false,
        context : [linkedId, contextAlignment[0], contextAlignment[1] ]
    });
    waitingOverlay.setBody(sBody);
    waitingOverlay.render(document.body);
    waitingOverlay.element.className += " jira-menubutton";
    waitingOverlay.linkedId = linkedId;

    var currentOverlay;
    this.reset = function()
    {
        currentOverlay = waitingOverlay;
        that.madeAjaxCall = false;
    }
    this.reset();
    var linkedE = document.getElementById(linkedId);

    /**
     * Shows the specified overlay, aligning correctly while doing this.
     *
     * YUI behaviour bug.  The alignment we have declared above only somes timees works
     * so we give it another kick in the guts and its works!
     */
    var showOverlay = function(overlay)
    {
        // On Safari, anchors dont have tab focus unless Steve Jobs or the user sets if specifically. So we keep a current track of a current anchor class
        YAHOO.util.Dom.addClass(overlay.linkedId, "current");

        // kick it again for alignment!
        overlay.align(contextAlignment[0], contextAlignment[1]);
        var linkRegion = YAHOO.util.Dom.getRegion(linkedId);
        var xy = YAHOO.util.Dom.getXY(overlay.element);
        if (xy[0] < 0 || xy[1] < linkRegion.bottom)
        {
            // normalize back to zero
            xy[0] = Math.max(xy[0], 0);
            xy[1] = Math.max(xy[1], linkRegion.bottom);
            YAHOO.util.Dom.setXY(overlay.element, xy);
        }

        //
        // It possible for the really long list to scroll up into the sky, past the top of the window.
        /// we restrict it to never going above the actual link
        //
        overlay.show();

        if( ! overlay.resizer ){
            overlay.resizer = function(){
                if( overlay.showing ){
                    window.setTimeout( function(){
                        showOverlay( overlay );
                    }, 350 );;
                }
            };
            YAHOO.util.Event.addListener(window,"resize", overlay.resizer );
        }
        overlay.showing = true;
    }

    var hideOverlay = function(overlay)
    {
        YAHOO.util.Dom.removeClass(overlay.linkedId, "current");
        overlay.hide();
        YAHOO.util.Event.removeListener(window,"resize", overlay.resizer );
        delete overlay.resizer;
    }

    /**
     * Toggles the current overlay
     */
    var toggleOverlay = function()
    {
        // if we are the current one, then we must be showing and hence a click is asking to close us
        if (jira.widget.MenuButton.current == currentOverlay)
        {
            hideOverlay(jira.widget.MenuButton.current);
            jira.widget.MenuButton.current = null;
        }
        else
        {
            if (jira.widget.MenuButton.current)
            {
                hideOverlay(jira.widget.MenuButton.current);
            }
            jira.widget.MenuButton.current = currentOverlay;
            showOverlay(jira.widget.MenuButton.current);
        }
    }

    var hideIfWeAreCurrent = function()
    {
        if (jira.widget.MenuButton.current == currentOverlay)
        {
            hideOverlay(jira.widget.MenuButton.current);
            jira.widget.MenuButton.current = null;
        }
    }

    /**
     * Our onclick handler
     */
    var onClick = function(ev)
    {
        // stop the propogation and default hanling of the click
        ev = YAHOO.util.Event.getEvent(ev);
        YAHOO.util.Event.stopEvent(ev);

        // have we any data
        if (! this.madeAjaxCall)
        {
            // now make an AJAX call for data
            this.madeAjaxCall = true;

            var escapeHTML = function(str)
            {
                var div = document.createElement('div');
                var text = document.createTextNode(str);
                div.appendChild(text);
                return div.innerHTML;
            }

            /**
             * Builds the YUI overlay with the specified data
             */
            var buildAndShowDataOverlay = function(sData)
            {
                var previousOverlay = currentOverlay;
                previousOverlay.destroy();

                // data has arrived
                var dataOverlay = new YAHOO.widget.Overlay(linkedId + "_data", {
                    constraintoviewport : true,
                    visible : false,
                    context : [linkedId, contextAlignment[0], contextAlignment[1] ]
                });
                dataOverlay.setBody(sData);
                dataOverlay.render(document.body);
                dataOverlay.element.className += " jira-menubutton";
                dataOverlay.linkedId = linkedId;

                if (jira.widget.MenuButton.current == previousOverlay)
                {
                    hideOverlay(jira.widget.MenuButton.current);
                    jira.widget.MenuButton.current = dataOverlay;
                    showOverlay(jira.widget.MenuButton.current);
                }
                // now the current menu is the data one
                currentOverlay = dataOverlay;
                //
                // a click handler that runs after the anchor link
                var onClickOnOverlay = function()
                {
                    //hideIfWeAreCurrent();
                }
                YAHOO.util.Event.addListener(currentOverlay.element, "mouseup", onClickOnOverlay);
                YAHOO.util.Event.removeListener(previousOverlay.element, "mouseup", onClickOnOverlay);
            }

            /**
             * Show a failure overlay in a worst case situation
             *
             * @param failureText
             */
            var buildAndShowFailure = function(ajaxResponse, failureText, exception)
            {
                var text = failureText + '<br/>' + ajaxResponse.status + ' - ' + ajaxResponse.statusText;
                if (exception)
                {
                    text += '<br/>' + exception.toString();
                }
                buildAndShowDataOverlay(text);
            }

            /**
             * Our AJAX callback handling
             */
            var ajaxCallback = {

                success : function(ajaxResponse)
                {
                    // it must be an array of objects in the form { text : xxx, url : yyyy }
                    // Note: an empty url denotes an item that should be displayed but not as a link
                    try
                    {
                        var jsonText = ajaxResponse.responseText;
                        var jsonObj = JSON.parse(jsonText);
                        var sData = '<ul>';
                        for (var i = 0; i < jsonObj.length; i++)
                        {
                            var item = jsonObj[i];
                            if (item && item.text)
                            {
                                var classStr = "";
                                if (item.groupmarker)
                                {
                                    classStr = ' class="groupmarker"';
                                }
                                sData += '<li' + classStr + '>';
                                sData += (item.url) ? '<a href="' + item.url + '">' : '<span>';
                                sData += escapeHTML(item.text);
                                sData += (item.url) ? '</a>' : '</span>';
                                sData += '</li>';
                            }
                        }
                        sData += "</ul>"
                        buildAndShowDataOverlay(sData);
                    }
                    catch (e)
                    {
                        buildAndShowFailure(ajaxResponse, "Unable to interpret data sent back from the JIRA server.</br>"
                                + "<ul>"
                                + "<li>Perhaps the user's session has timed out?</li>"
                                + "<li>Perhaps the user does not have JIRA 'use' permission?</li>"
                                + "</ul>", e);
                    }

                },

                failure : function(ajaxResponse)
                {
                    buildAndShowFailure(ajaxResponse, "Unable to contact the JIRA server");
                }
            }

            // make an ajax call for the data
            var paramsep = (url.indexOf("?") == -1) ? "?" : "&";
            var jsonUrl = url + paramsep + "json=true&decorator=none";
            var makeCallBack = function()
            {
                YAHOO.util.Connect.asyncRequest('GET', jsonUrl, ajaxCallback);
            }
            //
            // Use this for testing how a slow system will respond
            makeCallBack();
            //window.setTimeout(makeCallBack, 2000);
        }
        toggleOverlay();
    }
    YAHOO.util.Event.addListener(linkedE, "click", onClick, this, true);

    var onTab = function( event )
    {
        // the blur even may fire before the click event of the child anchor items
        // gets a chance to do its stuff.  So do lets not hide straight away and
        // prevent a click on a child anchor from registering
        if( event.keyCode == 9 ){
            hideIfWeAreCurrent();
        }
    }
    YAHOO.util.Event.addListener(linkedE, "keydown", onTab, this, true);


    // once only registration of body listeners
    if (! jira.widget.MenuButton.registeredClickAway)
    {
        jira.widget.MenuButton.registeredClickAway = false;
        var onClickAway = function()
        {
            if (jira.widget.MenuButton.current)
            {
                hideOverlay(jira.widget.MenuButton.current);
                jira.widget.MenuButton.current = null;
            }
        }
        YAHOO.util.Event.addListener(document.body, "click", onClickAway);
    }

    // register ourselves in the map of drop down menus
    jira.widget.MenuButton.menuButtonMap[linkedId] = this;
}

jira.widget.MenuButton.registeredClickAway = false;
jira.widget.MenuButton.current = null;
/**
 * A map of registered drop down menus
 */
jira.widget.MenuButton.menuButtonMap = {};

/**
 * This will invalidate the menuButton associated with the specified
 * id and cause it to go to the server again when shown
 *
 * @param linkedId
 */
jira.widget.MenuButton.invalidateMenuButton = function(linkedId)
{
    var menuButton = jira.widget.MenuButton.menuButtonMap[linkedId];
    if (menuButton)
    {
        menuButton.reset();
    }
}
